@@ -14,14 +14,15 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 import abc
 import ast
+import contextlib
 import copy
 
 from oslo_serialization import jsonutils
+import requests
 import six
-import six.moves.urllib.parse as urlparse
-import six.moves.urllib.request as urlrequest
 
 
 registered_checks = {}
@@ -77,13 +78,6 @@ class TrueCheck(BaseCheck):
 
 
 class Check(BaseCheck):
-    """A base class to allow for user-defined policy checks.
-
-    :param kind: The kind of the check, i.e., the field before the ``:``.
-    :param match: The match of the check, i.e., the field after the ``:``.
-
-    """
-
     def __init__(self, kind, match):
         self.kind = kind
         self.match = match
@@ -95,14 +89,6 @@ class Check(BaseCheck):
 
 
 class NotCheck(BaseCheck):
-    """Implements the "not" logical operator.
-
-    A policy check that inverts the result of another policy check.
-
-    :param rule: The rule to negate.  Must be a Check.
-
-    """
-
     def __init__(self, rule):
         self.rule = rule
 
@@ -121,14 +107,6 @@ class NotCheck(BaseCheck):
 
 
 class AndCheck(BaseCheck):
-    """Implements the "and" logical operator.
-
-    A policy check that requires that a list of other checks all return True.
-
-    :param list rules: rules that will be tested.
-
-    """
-
     def __init__(self, rules):
         self.rules = rules
 
@@ -164,15 +142,6 @@ class AndCheck(BaseCheck):
 
 
 class OrCheck(BaseCheck):
-    """Implements the "or" operator.
-
-    A policy check that requires that at least one of a list of other
-    checks returns ``True``.
-
-    :param rules: A list of rules that will be tested.
-
-    """
-
     def __init__(self, rules):
         self.rules = rules
 
@@ -202,19 +171,18 @@ class OrCheck(BaseCheck):
         self.rules.append(rule)
         return self
 
+    def pop_check(self):
+        """Pops the last check from the list and returns them
+
+        :returns: self, the popped check
+        :rtype: :class:`.OrCheck`, class:`.Check`
+        """
+
+        check = self.rules.pop()
+        return self, check
+
 
 def register(name, func=None):
-    """Register a function or :class:`.Check` class as a policy check.
-
-    :param name: Gives the name of the check type, e.g., "rule",
-                 "role", etc.  If name is ``None``, a default check type
-                 will be registered.
-    :param func: If given, provides the function or class to register.
-                 If not given, returns a function taking one argument
-                 to specify the function or class to register,
-                 allowing use as a decorator.
-    """
-
     # Perform the actual decoration by registering the function or
     # class.  Returns the function or class for compliance with the
     # decorator interface.
@@ -231,8 +199,6 @@ def register(name, func=None):
 
 @register('rule')
 class RuleCheck(Check):
-    """Recursively checks credentials based on the defined rules."""
-
     def __call__(self, target, creds, enforcer):
         try:
             return enforcer.rules[self.match](target, creds, enforcer)
@@ -246,7 +212,15 @@ class RoleCheck(Check):
     """Check that there is a matching role in the ``creds`` dict."""
 
     def __call__(self, target, creds, enforcer):
-        return self.match.lower() in [x.lower() for x in creds['roles']]
+        try:
+            match = self.match % target
+        except KeyError:
+            # While doing RoleCheck if key not
+            # present in Target return false
+            return False
+        if 'roles' in creds:
+            return match.lower() in [x.lower() for x in creds['roles']]
+        return False
 
 
 @register('http')
@@ -268,12 +242,10 @@ class HttpCheck(Check):
             element = target.get(key)
             if type(element) is object:
                 temp_target[key] = {}
-
         data = {'target': jsonutils.dumps(temp_target),
                 'credentials': jsonutils.dumps(creds)}
-        post_data = urlparse.urlencode(data)
-        f = urlrequest.urlopen(url, post_data)
-        return f.read() == 'True'
+        with contextlib.closing(requests.post(url, data=data)) as r:
+            return r.text == 'True'
 
 
 @register(None)
