@@ -20,23 +20,93 @@ Common Policy Engine Implementation
 
 Policies are expressed as a target and an associated rule::
 
-    "<target>": "<rule>"
+    "<target>": <rule>
 
 The `target` is specific to the service that is conducting policy
 enforcement.  Typically, the target refers to an API call.
 
-A rule is made up of zero or more checks, where zero checks will always
-allow the action that is being enforced.  A number of different check
-types are supported, which can be divided into generic checks and
-special checks.
+For the `<rule>` part, see `Policy Rule Expressions`.
+
+Policy Rule Expressions
+~~~~~~~~~~~~~~~~~~~~~~~
+
+Policy rules can be expressed in one of two forms: a string written in the new
+policy language or a list of lists. The string format is preferred since it's
+easier for most people to understand.
+
+In the policy language, each check is specified as a simple "a:b" pair that is
+matched to the correct class to perform that check:
+
+ +--------------------------------+------------------------------------------+
+ |            TYPE                |                SYNTAX                    |
+ +================================+==========================================+
+ |User's Role                     |              role:admin                  |
+ +--------------------------------+------------------------------------------+
+ |Rules already defined on policy |          rule:admin_required             |
+ +--------------------------------+------------------------------------------+
+ |Against URLs¹                   |         http://my-url.org/check          |
+ +--------------------------------+------------------------------------------+
+ |User attributes²                |    project_id:%(target.project.id)s      |
+ +--------------------------------+------------------------------------------+
+ |Strings                         |        - <variable>:'xpto2035abc'        |
+ |                                |        - 'myproject':<variable>          |
+ +--------------------------------+------------------------------------------+
+ |                                |         - project_id:xpto2035abc         |
+ |Literals                        |         - domain_id:20                   |
+ |                                |         - True:%(user.enabled)s          |
+ +--------------------------------+------------------------------------------+
+
+¹URL checking must return ``True`` to be valid
+
+²User attributes (obtained through the token): user_id, domain_id or project_id
+
+Conjunction operators ``and`` and ``or`` are available, allowing for more
+expressiveness in crafting policies. For example::
+
+    "role:admin or (project_id:%(project_id)s and role:projectadmin)"
+
+The policy language also has the ``not`` operator, allowing a richer
+policy rule::
+
+    "project_id:%(project_id)s and not role:dunce"
+
+Operator precedence is below:
+
+ +------------+-------------+-------------+
+ | PRECEDENCE |     TYPE    | EXPRESSION  |
+ +============+=============+=============+
+ |      4     |  Grouping   |    (...)    |
+ +------------+-------------+-------------+
+ |      3     | Logical NOT |   not ...   |
+ +------------+-------------+-------------+
+ |      2     | Logical AND | ... and ... |
+ +------------+-------------+-------------+
+ |      1     | Logical OR  | ... or ...  |
+ +------------+-------------+-------------+
+
+Operator with larger precedence number precedes others with smaller numbers.
+
+In the list-of-lists representation, each check inside the innermost
+list is combined as with an "and" conjunction -- for that check to pass,
+all the specified checks must pass.  These innermost lists are then
+combined as with an "or" conjunction. As an example, take the following
+rule, expressed in the list-of-lists representation::
+
+    [["role:admin"], ["project_id:%(project_id)s", "role:projectadmin"]]
+
+Finally, two special policy checks should be mentioned; the policy
+check "@" will always accept an access, and the policy check "!" will
+always reject an access.  (Note that if a rule is either the empty
+list (``[]``) or the empty string (``""``), this is equivalent to the "@"
+policy check.)  Of these, the "!" policy check is probably the most useful,
+as it allows particular rules to be explicitly disabled.
 
 Generic Checks
 ~~~~~~~~~~~~~~
 
-A :class:`generic check <oslo_policy.policy.GenericCheck>` is used
-to perform matching against attributes that are sent along with the API
-calls.  These attributes can be used by the policy engine (on the right
-side of the expression), by using the following syntax::
+A `generic` check is used to perform matching against attributes that are sent
+along with the API calls.  These attributes can be used by the policy engine
+(on the right side of the expression), by using the following syntax::
 
     <some_attribute>:%(user.id)s
 
@@ -81,9 +151,8 @@ checks.
 Role Check
 ^^^^^^^^^^
 
-A :class:`role check <oslo_policy.policy.RoleCheck>` is used to
-check if a specific role is present in the supplied credentials.  A role
-check is expressed as::
+A ``role`` check is used to check if a specific role is present in the supplied
+credentials.  A role check is expressed as::
 
     "role:<role_name>"
 
@@ -94,7 +163,7 @@ A :class:`rule check <oslo_policy.policy.RuleCheck>` is used to
 reference another defined rule by its name.  This allows for common
 checks to be defined once as a reusable rule, which is then referenced
 within other rules.  It also allows one to define a set of checks as a
-more descriptive name to aid in readabilty of policy.  A rule check is
+more descriptive name to aid in readability of policy.  A rule check is
 expressed as::
 
     "rule:<rule_name>"
@@ -108,11 +177,10 @@ which is then used via a rule check::
 HTTP Check
 ^^^^^^^^^^
 
-An :class:`http check <oslo_policy.policy.HttpCheck>` is used to
-make an HTTP request to a remote server to determine the results of the
-check.  The target and credentials are passed to the remote server for
-evaluation.  The action is authorized if the remote server returns a
-response of ``True``. An http check is expressed as::
+An ``http`` check is used to make an HTTP request to a remote server to
+determine the results of the check.  The target and credentials are passed to
+the remote server for evaluation.  The action is authorized if the remote
+server returns a response of ``True``. An http check is expressed as::
 
     "http:<target URI>"
 
@@ -136,68 +204,6 @@ The following classes can be used as parents for custom special check types:
     * :class:`~oslo_policy.policy.OrCheck`
     * :class:`~oslo_policy.policy.RuleCheck`
 
-Policy Rule Expressions
-~~~~~~~~~~~~~~~~~~~~~~~
-
-Policy rules can be expressed in one of two forms: A list of lists, or a
-string written in the new policy language.
-
-In the list-of-lists representation, each check inside the innermost
-list is combined as with an "and" conjunction--for that check to pass,
-all the specified checks must pass.  These innermost lists are then
-combined as with an "or" conjunction. As an example, take the following
-rule, expressed in the list-of-lists representation::
-
-    [["role:admin"], ["project_id:%(project_id)s", "role:projectadmin"]]
-
-This is the original way of expressing policies, but there now exists a
-new way: the policy language.
-
-In the policy language, each check is specified the same way as in the
-list-of-lists representation: a simple "a:b" pair that is matched to
-the correct class to perform that check:
-
- +--------------------------------+------------------------------------------+
- |            TYPE                |                SYNTAX                    |
- +================================+==========================================+
- |User's Role                     |              role:admin                  |
- +--------------------------------+------------------------------------------+
- |Rules already defined on policy |          rule:admin_required             |
- +--------------------------------+------------------------------------------+
- |Against URLs¹                   |         http://my-url.org/check          |
- +--------------------------------+------------------------------------------+
- |User attributes²                |    project_id:%(target.project.id)s      |
- +--------------------------------+------------------------------------------+
- |Strings                         |        - <variable>:'xpto2035abc'        |
- |                                |        - 'myproject':<variable>          |
- +--------------------------------+------------------------------------------+
- |                                |         - project_id:xpto2035abc         |
- |Literals                        |         - domain_id:20                   |
- |                                |         - True:%(user.enabled)s          |
- +--------------------------------+------------------------------------------+
-
-¹URL checking must return ``True`` to be valid
-
-²User attributes (obtained through the token): user_id, domain_id or project_id
-
-Conjunction operators are available, allowing for more expressiveness
-in crafting policies. So, in the policy language, the previous check in
-list-of-lists becomes::
-
-    role:admin or (project_id:%(project_id)s and role:projectadmin)
-
-The policy language also has the ``not`` operator, allowing a richer
-policy rule::
-
-    project_id:%(project_id)s and not role:dunce
-
-Finally, two special policy checks should be mentioned; the policy
-check "@" will always accept an access, and the policy check "!" will
-always reject an access.  (Note that if a rule is either the empty
-list ("[]") or the empty string, this is equivalent to the "@" policy
-check.)  Of these, the "!" policy check is probably the most useful,
-as it allows particular rules to be explicitly disabled.
-
 Default Rule
 ~~~~~~~~~~~~
 
@@ -227,12 +233,56 @@ LOG = logging.getLogger(__name__)
 
 
 register = _checks.register
+"""Register a function or :class:`.Check` class as a policy check.
+
+:param name: Gives the name of the check type, e.g., "rule",
+             "role", etc.  If name is ``None``, a default check type
+             will be registered.
+:param func: If given, provides the function or class to register.
+             If not given, returns a function taking one argument
+             to specify the function or class to register,
+             allowing use as a decorator.
+"""
+
 Check = _checks.Check
+"""A base class to allow for user-defined policy checks.
+
+:param kind: The kind of the check, i.e., the field before the ``:``.
+:param match: The match of the check, i.e., the field after the ``:``.
+
+"""
 
 AndCheck = _checks.AndCheck
+"""Implements the "and" logical operator.
+
+A policy check that requires that a list of other checks all return True.
+
+:param list rules: rules that will be tested.
+
+"""
+
 NotCheck = _checks.NotCheck
+"""Implements the "not" logical operator.
+
+A policy check that inverts the result of another policy check.
+
+:param rule: The rule to negate.
+:type rule: oslo_policy.policy.Check
+
+"""
+
 OrCheck = _checks.OrCheck
+"""Implements the "or" operator.
+
+A policy check that requires that at least one of a list of other
+checks returns ``True``.
+
+:param rules: A list of rules that will be tested.
+
+"""
+
 RuleCheck = _checks.RuleCheck
+"""Recursively checks credentials based on the defined rules."""
 
 
 class PolicyNotAuthorized(Exception):
@@ -252,8 +302,8 @@ class Rules(dict):
         """Allow loading of JSON rule data."""
 
         # Suck in the JSON data and parse the rules
-        rules = dict((k, _parser.parse_rule(v)) for k, v in
-                     jsonutils.loads(data).items())
+        rules = {k: _parser.parse_rule(v)
+                 for k, v in jsonutils.loads(data).items()}
 
         return cls(rules, default_rule)
 
@@ -262,7 +312,7 @@ class Rules(dict):
         """Allow loading of rule data from a dictionary."""
 
         # Parse the rules stored in the dictionary
-        rules = dict((k, _parser.parse_rule(v)) for k, v in rules_dict.items())
+        rules = {k: _parser.parse_rule(v) for k, v in rules_dict.items()}
 
         return cls(rules, default_rule)
 
@@ -425,6 +475,8 @@ class Enforcer(object):
 
     @staticmethod
     def _walk_through_policy_directory(path, func, *args):
+        if not os.path.isdir(path):
+            raise ValueError('%s is not a directory' % path)
         # We do not iterate over sub-directories.
         policy_files = next(os.walk(path))[2]
         policy_files.sort()
