@@ -13,13 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
 import mock
 from oslo_serialization import jsonutils
 from oslotest import base as test_base
-import six
+from requests_mock.contrib import fixture as rm_fixture
 import six.moves.urllib.parse as urlparse
-import six.moves.urllib.request as urlrequest
 
 from oslo_policy import _checks
 from oslo_policy.tests import base
@@ -74,61 +72,75 @@ class RoleCheckTestCase(base.PolicyBaseTestCase):
     def test_accept(self):
         check = _checks.RoleCheck('role', 'sPaM')
 
-        self.assertTrue(check('target', dict(roles=['SpAm']), self.enforcer))
+        self.assertTrue(check({}, dict(roles=['SpAm']), self.enforcer))
 
     def test_reject(self):
         check = _checks.RoleCheck('role', 'spam')
 
-        self.assertFalse(check('target', dict(roles=[]), self.enforcer))
+        self.assertFalse(check({}, dict(roles=[]), self.enforcer))
+
+    def test_format_value(self):
+        check = _checks.RoleCheck('role', '%(target.role.name)s')
+
+        target_dict = {'target.role.name': 'a'}
+        cred_dict = dict(user='user', roles=['a', 'b', 'c'])
+        self.assertTrue(check(target_dict, cred_dict, self.enforcer))
+
+        target_dict = {'target.role.name': 'd'}
+        self.assertFalse(check(target_dict, cred_dict, self.enforcer))
+
+        target_dict = dict(target=dict(role=dict()))
+        self.assertFalse(check(target_dict, cred_dict, self.enforcer))
+
+    def test_no_roles_case(self):
+        check = _checks.RoleCheck('role', 'spam')
+
+        self.assertFalse(check({}, {}, self.enforcer))
 
 
 class HttpCheckTestCase(base.PolicyBaseTestCase):
+
+    def setUp(self):
+        super(HttpCheckTestCase, self).setUp()
+        self.requests_mock = self.useFixture(rm_fixture.Fixture())
+
     def decode_post_data(self, post_data):
         result = {}
         for item in post_data.split('&'):
             key, _sep, value = item.partition('=')
             result[key] = jsonutils.loads(urlparse.unquote_plus(value))
-
         return result
 
-    @mock.patch.object(urlrequest, 'urlopen',
-                       return_value=six.StringIO('True'))
-    def test_accept(self, mock_urlopen):
-        check = _checks.HttpCheck('http', '//example.com/%(name)s')
-        self.assertTrue(check(dict(name='target', spam='spammer'),
-                              dict(user='user', roles=['a', 'b', 'c']),
-                              self.enforcer))
-        self.assertEqual(1, mock_urlopen.call_count)
+    def test_accept(self):
+        self.requests_mock.post('http://example.com/target', text='True')
 
-        args = mock_urlopen.call_args[0]
-
-        self.assertEqual('http://example.com/target', args[0])
-        self.assertEqual(dict(
-            target=dict(name='target', spam='spammer'),
-            credentials=dict(user='user', roles=['a', 'b', 'c']),
-        ), self.decode_post_data(args[1]))
-
-    @mock.patch.object(urlrequest, 'urlopen',
-                       return_value=six.StringIO('other'))
-    def test_reject(self, mock_urlopen):
         check = _checks.HttpCheck('http', '//example.com/%(name)s')
 
-        self.assertFalse(check(dict(name='target', spam='spammer'),
-                               dict(user='user', roles=['a', 'b', 'c']),
-                               self.enforcer))
-        self.assertEqual(1, mock_urlopen.call_count)
+        target_dict = dict(name='target', spam='spammer')
+        cred_dict = dict(user='user', roles=['a', 'b', 'c'])
+        self.assertTrue(check(target_dict, cred_dict, self.enforcer))
 
-        args = mock_urlopen.call_args[0]
+        last_request = self.requests_mock.last_request
+        self.assertEqual('POST', last_request.method)
+        self.assertEqual(dict(target=target_dict, credentials=cred_dict),
+                         self.decode_post_data(last_request.body))
 
-        self.assertEqual('http://example.com/target', args[0])
-        self.assertEqual(dict(
-            target=dict(name='target', spam='spammer'),
-            credentials=dict(user='user', roles=['a', 'b', 'c']),
-        ), self.decode_post_data(args[1]))
+    def test_reject(self):
+        self.requests_mock.post("http://example.com/target", text='other')
 
-    @mock.patch.object(urlrequest, 'urlopen',
-                       return_value=six.StringIO('True'))
-    def test_http_with_objects_in_target(self, mock_urlopen):
+        check = _checks.HttpCheck('http', '//example.com/%(name)s')
+
+        target_dict = dict(name='target', spam='spammer')
+        cred_dict = dict(user='user', roles=['a', 'b', 'c'])
+        self.assertFalse(check(target_dict, cred_dict, self.enforcer))
+
+        last_request = self.requests_mock.last_request
+        self.assertEqual('POST', last_request.method)
+        self.assertEqual(dict(target=target_dict, credentials=cred_dict),
+                         self.decode_post_data(last_request.body))
+
+    def test_http_with_objects_in_target(self):
+        self.requests_mock.post("http://example.com/target", text='True')
 
         check = _checks.HttpCheck('http', '//example.com/%(name)s')
         target = {'a': object(),
@@ -138,9 +150,9 @@ class HttpCheckTestCase(base.PolicyBaseTestCase):
                               dict(user='user', roles=['a', 'b', 'c']),
                               self.enforcer))
 
-    @mock.patch.object(urlrequest, 'urlopen',
-                       return_value=six.StringIO('True'))
-    def test_http_with_strings_in_target(self, mock_urlopen):
+    def test_http_with_strings_in_target(self):
+        self.requests_mock.post("http://example.com/target", text='True')
+
         check = _checks.HttpCheck('http', '//example.com/%(name)s')
         target = {'a': 'some_string',
                   'name': 'target',
@@ -395,6 +407,13 @@ class OrCheckTestCase(test_base.BaseTestCase):
 
         self.assertEqual(['rule1', 'rule2', 'rule3'], check.rules)
 
+    def test_pop_check(self):
+        check = _checks.OrCheck(['rule1', 'rule2', 'rule3'])
+        rules, check1 = check.pop_check()
+
+        self.assertEqual(['rule1', 'rule2'], check.rules)
+        self.assertEqual('rule3', check1)
+
     def test_str(self):
         check = _checks.OrCheck(['rule1', 'rule2'])
 
