@@ -30,30 +30,13 @@ from oslo_policy import policy
 from oslo_policy.tests import base
 
 
-POLICY_A_CONTENTS = """
-{
-    "default": "role:fakeA"
-}
-"""
-
-POLICY_B_CONTENTS = """
-{
-  "default": "role:fakeB"
-}
-"""
-
-POLICY_FAKE_CONTENTS = """
-{
-    "default": "role:fakeC"
-}
-"""
-
-POLICY_JSON_CONTENTS = """
-{
+POLICY_A_CONTENTS = jsonutils.dumps({"default": "role:fakeA"})
+POLICY_B_CONTENTS = jsonutils.dumps({"default": "role:fakeB"})
+POLICY_FAKE_CONTENTS = jsonutils.dumps({"default": "role:fakeC"})
+POLICY_JSON_CONTENTS = jsonutils.dumps({
     "default": "rule:admin",
     "admin": "is_admin:True"
-}
-"""
+})
 
 
 class MyException(Exception):
@@ -100,10 +83,10 @@ class RulesTestCase(test_base.BaseTestCase):
 
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_load_json(self):
-        exemplar = """{
-    "admin_or_owner": [["role:admin"], ["project_id:%(project_id)s"]],
-    "default": []
-}"""
+        exemplar = jsonutils.dumps({
+            "admin_or_owner": [["role:admin"], ["project_id:%(project_id)s"]],
+            "default": []
+        })
         rules = policy.Rules.load_json(exemplar, 'default')
 
         self.assertEqual('default', rules.default_rule)
@@ -112,6 +95,17 @@ class RulesTestCase(test_base.BaseTestCase):
             default=[],
         ), rules)
 
+    @mock.patch.object(_parser, 'parse_rule', lambda x: x)
+    def test_load_json_invalid_exc(self):
+        # When the JSON isn't valid, ValueError is raised on load_json.
+        # Note the trailing , in the exemplar is invalid JSON.
+        exemplar = """{
+    "admin_or_owner": [["role:admin"], ["project_id:%(project_id)s"]],
+    "default": [],
+}"""
+        self.assertRaises(ValueError, policy.Rules.load_json, exemplar,
+                          'default')
+
     @mock.patch.object(_parser, 'parse_rule', lambda x: x)
     def test_from_dict(self):
         expected = {'admin_or_owner': 'role:admin', 'default': '@'}
@@ -121,9 +115,9 @@ class RulesTestCase(test_base.BaseTestCase):
         self.assertEqual(expected, rules)
 
     def test_str(self):
-        exemplar = """{
-    "admin_or_owner": "role:admin or project_id:%(project_id)s"
-}"""
+        exemplar = jsonutils.dumps({
+            "admin_or_owner": "role:admin or project_id:%(project_id)s"
+        }, indent=4)
         rules = policy.Rules(dict(
             admin_or_owner='role:admin or project_id:%(project_id)s',
         ))
@@ -131,9 +125,9 @@ class RulesTestCase(test_base.BaseTestCase):
         self.assertEqual(exemplar, str(rules))
 
     def test_str_true(self):
-        exemplar = """{
-    "admin_or_owner": ""
-}"""
+        exemplar = jsonutils.dumps({
+            "admin_or_owner": ""
+        }, indent=4)
         rules = policy.Rules(dict(
             admin_or_owner=_checks.TrueCheck(),
         ))
@@ -260,6 +254,13 @@ class EnforcerTest(base.PolicyBaseTestCase):
         self.assertIn('admin', self.enforcer.rules)
         self.check_loaded_files(['policy.json', 'policy.d/a.conf'])
 
+    def test_load_policy_dirs_with_non_directory(self):
+        self.create_config_file('policy.d/a.conf', POLICY_A_CONTENTS)
+        self.conf.set_override('policy_dirs',
+                               ['policy.d/a.conf'],
+                               group='oslo_policy')
+        self.assertRaises(ValueError, self.enforcer.load_rules, True)
+
     def test_set_rules_type(self):
         self.assertRaises(TypeError,
                           self.enforcer.set_rules,
@@ -271,14 +272,14 @@ class EnforcerTest(base.PolicyBaseTestCase):
         self.enforcer.rules = 'spam'
         self.enforcer.clear()
         self.assertEqual({}, self.enforcer.rules)
-        self.assertEqual(None, self.enforcer.default_rule)
-        self.assertEqual(None, self.enforcer.policy_path)
+        self.assertIsNone(self.enforcer.default_rule)
+        self.assertIsNone(self.enforcer.policy_path)
 
     def test_rule_with_check(self):
-        rules_json = """{
-                        "deny_stack_user": "not role:stack_user",
-                        "cloudwatch:PutMetricData": ""
-                        }"""
+        rules_json = jsonutils.dumps({
+            "deny_stack_user": "not role:stack_user",
+            "cloudwatch:PutMetricData": ""
+        })
         rules = policy.Rules.load_json(rules_json)
         self.enforcer.set_rules(rules)
         action = 'cloudwatch:PutMetricData'
@@ -286,10 +287,10 @@ class EnforcerTest(base.PolicyBaseTestCase):
         self.assertTrue(self.enforcer.enforce(action, {}, creds))
 
     def test_enforcer_with_default_rule(self):
-        rules_json = """{
-                        "deny_stack_user": "not role:stack_user",
-                        "cloudwatch:PutMetricData": ""
-                        }"""
+        rules_json = jsonutils.dumps({
+            "deny_stack_user": "not role:stack_user",
+            "cloudwatch:PutMetricData": ""
+        })
         rules = policy.Rules.load_json(rules_json)
         default_rule = _checks.TrueCheck()
         enforcer = policy.Enforcer(self.conf, default_rule=default_rule)
@@ -470,18 +471,32 @@ class CheckFunctionTestCase(base.PolicyBaseTestCase):
 
         self.assertEqual(('target', 'creds', self.enforcer), result)
 
-    def test_check_raises(self):
+    def test_check_rule_not_exist_not_empty_policy_file(self):
+        # If the rule doesn't exist, then enforce() fails rather than KeyError.
+
+        # This test needs a non-empty file otherwise the code short-circuits.
+        self.create_config_file('policy.json', jsonutils.dumps({"a_rule": []}))
+        self.enforcer.default_rule = None
+        self.enforcer.load_rules()
+        result = self.enforcer.enforce('rule', 'target', 'creds')
+        self.assertFalse(result)
+
+    def test_check_raise_default(self):
+        # When do_raise=True and exc is not used then PolicyNotAuthorized is
+        # raised.
         self.enforcer.set_rules(dict(default=_checks.FalseCheck()))
 
-        try:
-            self.enforcer.enforce('rule', 'target', 'creds',
-                                  True, MyException, 'arg1',
-                                  'arg2', kw1='kwarg1', kw2='kwarg2')
-        except MyException as exc:
-            self.assertEqual(('arg1', 'arg2'), exc.args)
-            self.assertEqual(dict(kw1='kwarg1', kw2='kwarg2'), exc.kwargs)
-        else:
-            self.fail('enforcer.enforce() failed to raise requested exception')
+        self.assertRaises(policy.PolicyNotAuthorized, self.enforcer.enforce,
+                          'rule', 'target', 'creds', True)
+
+    def test_check_raise_custom_exception(self):
+        self.enforcer.set_rules(dict(default=_checks.FalseCheck()))
+
+        exc = self.assertRaises(
+            MyException, self.enforcer.enforce, 'rule', 'target', 'creds',
+            True, MyException, 'arg1', 'arg2', kw1='kwarg1', kw2='kwarg2')
+        self.assertEqual(('arg1', 'arg2'), exc.args)
+        self.assertEqual(dict(kw1='kwarg1', kw2='kwarg2'), exc.kwargs)
 
 
 class RegisterCheckTestCase(base.PolicyBaseTestCase):
